package me.hammerle.supersnuvi.entity;

import me.hammerle.supersnuvi.entity.components.DefaultHealth;
import me.hammerle.supersnuvi.entity.components.DefaultMovement;
import me.hammerle.supersnuvi.entity.components.Death;
import me.hammerle.supersnuvi.entity.components.DefaultDespawn;
import me.hammerle.supersnuvi.entity.components.DefaultEnergy;
import me.hammerle.supersnuvi.entity.components.Health;
import me.hammerle.supersnuvi.entity.components.Movement;
import me.hammerle.supersnuvi.entity.components.HeroItemCollector;
import me.hammerle.supersnuvi.entity.components.LandMovement;
import me.hammerle.supersnuvi.entity.components.MovementPenalty;
import me.hammerle.supersnuvi.entity.components.StoneMovement;
import me.hammerle.supersnuvi.entity.components.ai.EntityController;
import me.hammerle.supersnuvi.entity.components.ai.HumanController;
import me.hammerle.supersnuvi.entity.components.ai.LondonerController;
import me.hammerle.supersnuvi.entity.components.ai.WalkController;
import me.hammerle.supersnuvi.entity.components.animator.StoneAnimator;
import me.hammerle.supersnuvi.entity.components.animator.HeroAnimator;
import me.hammerle.supersnuvi.entity.components.animator.EntityAnimator;
import me.hammerle.supersnuvi.entity.components.animator.LondonerAnimator;
import me.hammerle.supersnuvi.gamelogic.Level;
import me.hammerle.supersnuvi.rendering.Game;

public final class EntityBuilder 
{
    public static EntityBuilder newBuilder(Level level, double x, double y, double width, double height, double cWidth, double cHeight)
    {
        return new EntityBuilder(new Entity(level, x, y, width, height, cWidth, cHeight));
    }
    
    public static Entity buildHero(Level level, double x, double y)
    {
        double w = Game.TILE_SIZE;
        double h = Game.TILE_SIZE * 2;
        
        Entity hero = new Entity(level, x, y, w, h, w * 0.90625, h * 0.921875);
        hero.animator = new HeroAnimator(hero);
        hero.controller = new HumanController(hero);
        hero.health = new DefaultHealth(hero, new Death() 
                {
                    @Override
                    public void onDeath(Entity ent) 
                    {
                        ent.getLevel().scheduleReset();
                    }
                }, 100.0);
        hero.energy = new DefaultEnergy(hero, 100.0);
        hero.move = new DefaultMovement(hero, ent -> level.getData().getInt("jump", 24));
        hero.itemCollector = new HeroItemCollector(hero);
        hero.movePenalty = new LandMovement(hero);
        return hero;
    }
    
    public static Entity buildTest(Level level, double x, double y)
    {
        double w = Game.TILE_SIZE;
        double h = Game.TILE_SIZE * 2;

        Entity hero = new Entity(level, x, y, w, h, w * 0.4375, h * 0.703125);
        hero.animator = new LondonerAnimator(hero);
        //hero.controller = new FollowHeroController(hero, 2);
        hero.controller = new LondonerController(hero, true);
        hero.health = new DefaultHealth(hero, Death.NULL, 100.0);
        hero.energy = new DefaultEnergy(hero, 100.0);
        hero.move = new DefaultMovement(hero, ent -> level.getData().getInt("jump", 24));
        //hero.itemCollector = new HeroItemCollector(hero);
        hero.movePenalty = new LandMovement(hero);
        return hero;
    }
    
    public static Entity buildCrumblingStone(Level level, double x, double y)
    {
        Entity stone = new Entity(level, x, y, Game.TILE_SIZE, Game.TILE_SIZE * 10);
        stone.animator = new StoneAnimator(stone);
        stone.move = new StoneMovement(stone);
        stone.health = new DefaultDespawn(stone);
        return stone;
    }
    
    public static Entity fromId(int id, Level level, double x, double y)
    {
        switch(id)
        {
            case 1:
                return buildTest(level, x, y);
            case 2:
            {
                Entity ent = new Entity(level, x, y, Game.TILE_SIZE, Game.TILE_SIZE * 2);
                ent.animator = new HeroAnimator(ent);
                ent.controller = new WalkController(ent, 1);
                ent.health = new DefaultHealth(ent, Death.NULL, 100.0);
                ent.move = new DefaultMovement(ent, enti -> 12);
                ent.movePenalty = new LandMovement(ent);
                return ent;
            }
        }
        return null;
    }
    
    
    
    private final Entity ent;
    
    private EntityBuilder(Entity ent)
    {
        this.ent = ent;
    }
    
    public EntityBuilder setController(EntityController controller)
    {
        ent.controller = controller;
        return this;
    }
    
    public EntityBuilder setAnimator(EntityAnimator animator)
    {
        ent.animator = animator;
        return this;
    }
    
    public EntityBuilder setHealth(Health health)
    {
        ent.health = health;
        return this;
    }
    
    public EntityBuilder setMovement(Movement move)
    {
        ent.move = move;
        return this;
    }
    
    public EntityBuilder setMovementPenalty(MovementPenalty move)
    {
        ent.movePenalty = move;
        return this;
    }
}
