package me.hammerle.supersnuvi.entity.components;

import me.hammerle.supersnuvi.entity.Entity;

public class DefaultHealth extends Health
{
    private final double maxHealth;
    private double health;
    
    private int hurtTicks = 0;
    private int invincibility = 0;
    
    private int deathTicks = -1;
    private boolean isDead = false;
    
    public DefaultHealth(Entity ent, Death death, double maxHealth) 
    {
        super(ent, death);
        this.maxHealth = maxHealth;
        this.health = maxHealth;
    }

    @Override
    public void tick() 
    {
        if(invincibility > 0)
        {
            invincibility--;
        }
        
        if(hurtTicks > 0)
        {
            hurtTicks--;
        }
        else if(hurtTicks < 0)
        {
            hurtTicks++;
        }
        
        if(deathTicks != -1)
        {
            deathTicks++;
            if(deathTicks > ent.getAnimator().getDeathTicks())
            {
                death.onDeath(ent);
                isDead = true;
            }
        }
        else if(ent.isOutOfMap())
        {
            death.onDeath(ent);
            isDead = true;
        }
    }

    @Override
    public boolean isDead() 
    {
        return deathTicks != -1;
    }
    
    @Override
    public boolean shouldDespawn() 
    {
        return isDead;
    }
    
    @Override
    public boolean wasHurt() 
    {
        return hurtTicks > 0;
    }

    @Override
    public boolean hasInvincibility() 
    {
        return invincibility > 0;
    }

    @Override
    public boolean wasHealed() 
    {
        return hurtTicks < 0;
    }

    @Override
    public double getMaxHealth() 
    {
        return maxHealth;
    }

    @Override
    public double getHealth() 
    {
        return health;
    }

    @Override
    public void addHealth(double h) 
    {
        if(h < 0)
        {
            if(invincibility > 0)
            {
                return;
            }
            invincibility = 100;
            ent.getAnimator().resetHeadFrames();
            hurtTicks = 15;
        }
        else
        {
            hurtTicks = -5;
        }
        health += h;
        if(health > maxHealth)
        {
            health = maxHealth;
        }
        else if(health < 0.0)
        {
            if(deathTicks == -1)
            {
                deathTicks = 0;
                ent.getAnimator().resetFrames();
            }
        }
    }
}
