package me.hammerle.supersnuvi.tiles;

import java.util.HashSet;
import me.hammerle.supersnuvi.javafx.Image;
import me.hammerle.supersnuvi.entity.Entity;
import me.hammerle.supersnuvi.gamelogic.StateRenderer;
import me.hammerle.supersnuvi.util.CollisionBox;
import me.hammerle.supersnuvi.util.Face;
import me.hammerle.supersnuvi.util.SoundUtils;
import me.hammerle.supersnuvi.util.Utils;

public class BottledSoulTile extends Tile
{
    public static final Image[] IMAGE = new Image[] 
    {
        Utils.getTileImage("bottled_soul/bottled_soul"),
        Utils.getTileImage("bottled_soul/brilliant_soul"),
        Utils.getTileImage("bottled_soul/queer_soul")
    };
    
    private final HashSet<Long> states;
    private final int score;
    
    public BottledSoulTile(int score)
    {
        states = new HashSet<>();
        super.setCollisionBox(CollisionBox.createScaledBox(0.2, 0.2, 0.8, 0.8));
        this.score = score;
    }

    @Override
    public void onEntityCollide(Entity ent, int x, int y, Face face) 
    {
        if(ent.getItemCollector().canCollect())
        {
            if(states.add(getKey(x, y)))
            {
                if(ent.getItemCollector().isHero())
                {
                    SoundUtils.playSound(SoundUtils.Sound.COLLECT);
                    ent.getLevel().increaseSouls(score);
                }
                ent.getHealth().addHealthPercent(0.143 * score);
            }
        }
    }
    
    @Override
    public Image getImage(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return StateRenderer.FALLBACK_TILE.getImage(x, y);
        }
        return IMAGE[score - 1];
    }
    
    @Override
    public void reset() 
    {
        states.clear();
    } 
    
    @Override
    public int getBottleScore()
    {
        return score;
    }
}
