package me.hammerle.supersnuvi.tiles;

import java.util.HashSet;
import me.hammerle.supersnuvi.javafx.Image;
import me.hammerle.supersnuvi.entity.Entity;
import me.hammerle.supersnuvi.entity.EntityBuilder;
import me.hammerle.supersnuvi.gamelogic.Level;
import me.hammerle.supersnuvi.gamelogic.StateRenderer;
import me.hammerle.supersnuvi.util.CollisionBox;
import me.hammerle.supersnuvi.util.Face;
import me.hammerle.supersnuvi.util.SoundUtils;
import me.hammerle.supersnuvi.util.Utils;

public class CrumblingStoneTile extends Tile
{
    private final Image image = Utils.getTileImage("crumbling_stones/crumbling_stones");
    
    private final HashSet<Long> states;
    
    public CrumblingStoneTile() 
    {
        states = new HashSet<>();
        super.setCollisionBox(CollisionBox.createScaledBox(0, 0.4, 1, 1));
        super.setMovementBox(CollisionBox.createScaledBox(0, 0.4, 1, 1));
    }

    @Override
    public CollisionBox getCollisionBox(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return CollisionBox.NULL_BOX;
        }
        return super.getCollisionBox(x, y);
    }

    @Override
    public CollisionBox getMovementBox(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return CollisionBox.NULL_BOX;
        }
        return super.getMovementBox(x, y);
    }

    @Override
    public void onEntityCollide(Entity ent, int x, int y, Face face) 
    {
        super.onEntityCollide(ent, x, y, face);
        if(face == Face.UP)
        {
            if(states.add(getKey(x, y)))
            {
                Level l = ent.getLevel();               
                l.spawnEntity(EntityBuilder.buildCrumblingStone(l, Utils.toCoord(x), Utils.toCoord(y - 9)));
                SoundUtils.playSound(SoundUtils.Sound.STONE_CRUMBLING);
            }
        }
    }

    @Override
    public Image getImage(int x, int y) 
    {
        if(states.contains(getKey(x, y)))
        {
            return StateRenderer.FALLBACK_TILE.getImage(x, y);
        }
        return image;
    }

    @Override
    public void reset() 
    {
        states.clear();
    } 
}
